@@ -28,6 +28,8 @@ import enum
 import dataclasses as dtc
 import inspect
 import logging
+import os
+import platform
 import typing as tp
 from collections.abc import Awaitable, AsyncIterator, AsyncGenerator
 from contextlib import asynccontextmanager
@@ -106,7 +108,7 @@ class StreamsClient(ConnectionManager[RabbitMQStreamsProtocol]):
             RabbitMQStreamsProtocol, host=cinfo.host, port=cinfo.port
         )
         logger.info('rabbitmq streams protocol created: {}'.format(cinfo))
-        return tp.cast(RabbitMQStreamsProtocol, proto)
+        return proto
 
     async def _connect(self, protocol: RabbitMQStreamsProtocol) -> None:
         """
@@ -122,7 +124,9 @@ class StreamsClient(ConnectionManager[RabbitMQStreamsProtocol]):
         logger.info('rabbitmq streams client connected: {}'.format(cinfo))
 
         for publisher in self._publishers.values():
-            await self._create_publisher(protocol, publisher.stream, publisher.id)
+            await self._create_publisher(
+                protocol, publisher.stream, publisher.id, publisher.name
+            )
 
         for subscriber in self._subscribers.values():
             msg = subscriber.message
@@ -167,22 +171,22 @@ class StreamsClient(ConnectionManager[RabbitMQStreamsProtocol]):
     # now, ideally, we want Mypy to accept the following signature
     #
     #     @asynccontextmanager
-    #     def publisher(self, stream: str, *, cls: type[T]=Publisher) -> AsyncIterator[T]:
+    #     def publisher(self, stream: str, *, name: str | None=None, cls: type[T]=Publisher) -> AsyncIterator[T]:
     #
     # unfortunately, we need to use the overload hack as suggested at
     #
     #     https://github.com/python/mypy/issues/3737
     #
     @tp.overload
-    def publisher(self, stream: str) -> tp.AsyncContextManager[Publisher]:
+    def publisher(self, stream: str, *, name: str | None=None) -> tp.AsyncContextManager[Publisher]:
         ...
 
     @tp.overload
-    def publisher(self, stream: str, *, cls: type[T]) -> tp.AsyncContextManager[T]:
+    def publisher(self, stream: str, *, name: str | None=None, cls: type[T]) -> tp.AsyncContextManager[T]:
         ...
 
     @asynccontextmanager
-    async def publisher(self, stream, *, cls=Publisher):  # type: ignore
+    async def publisher(self, stream, *, name: str | None=None, cls=Publisher):  # type: ignore
         """
         Create publisher for RabbitMQ stream.
 
@@ -190,7 +194,13 @@ class StreamsClient(ConnectionManager[RabbitMQStreamsProtocol]):
 
         The stream must exist.
 
+        Publisher reference name is used for deduplication of messages. By
+        default, the publisher name is ``<hostname>/<pid>``. Override it, if
+        this scheme does not work for a specific application, i.e. when
+        using threads.
+
         :param stream: RabbitMQ stream name.
+        :param name: RabbitMQ stream publisher reference name.
         :param cls: Publisher class.
 
         .. seealso::
@@ -205,8 +215,13 @@ class StreamsClient(ConnectionManager[RabbitMQStreamsProtocol]):
 
         protocol = await self.get_protocol()
 
-        msg_id = await self._create_publisher(protocol, stream, publisher_id)
-        pub = cls(self, stream, publisher_id, msg_id + 1)
+        if name is None:
+            name = '{}/{}'.format(platform.node(), os.getpid())
+
+        msg_id = await self._create_publisher(
+            protocol, stream, publisher_id, name
+        )
+        pub = cls(self, stream, publisher_id, name, msg_id + 1)
         self._publishers[publisher_id] = pub
         try:
             yield self._publishers[publisher_id]
@@ -246,17 +261,17 @@ class StreamsClient(ConnectionManager[RabbitMQStreamsProtocol]):
 
     async def _create_publisher(
             self,
-            protocol: RabbitMQStreamsProtocol, stream: str, publisher_id: int
+            protocol: RabbitMQStreamsProtocol,
+            stream: str,
+            publisher_id: int,
+            name: str,
         ) -> int:
         # TODO: what is the correlation between publisher reference string
         # and publisher id?
-        publisher_ref = 'publisher_{}_{:03d}'.format(stream, publisher_id)
-        msg_id = await protocol.create_publisher(
-            publisher_id, publisher_ref, stream
-        )
+        msg_id = await protocol.create_publisher(publisher_id, name, stream)
         logger.info(
-            'publisher (re)created, ref={}, stream={}, message id={}'
-            .format(publisher_ref, stream, msg_id)
+            'publisher (re)created, name={}, stream={}, message id={}'
+            .format(name, stream, msg_id)
         )
         return msg_id
 
