@@ -219,7 +219,7 @@ class RabbitMQStreamsProtocol(asyncio.Protocol):
         Create RabbitMQ Streams publisher and return last message id.
 
         :param publisher_id: Publisher id.
-        :param publisher_ref: Publisher reference string.
+        :param publisher_ref: Publisher reference name.
         :param stream: RabbitMQ stream name.
 
         .. seealso::
@@ -502,6 +502,10 @@ class RabbitMQStreamsProtocol(asyncio.Protocol):
         for t in tasks:
             t.set_exception(ConnectionError('connection closed'))
 
+        self._requests.clear()
+        self._published_messages.clear()
+        self._subscriptions.clear()
+
         logger.info('connection is closed')
 
     def abort(self) -> None:
@@ -619,7 +623,16 @@ class RabbitMQStreamsProtocol(asyncio.Protocol):
             )
         )
 
-        task = self._requests.pop(correlation_id)
+        task = self._requests.pop(correlation_id, None)
+        if not task:
+            # maybe wrong response from the broker, maybe task no longer
+            # valid due to disconnection process in progress
+            logger.warning(
+                'request task not found, correlation id={}'
+                .format(correlation_id)
+            )
+            return
+
         if code == 1 and key == const.RESPONSE_KEY_QUERY_PUBLISHER_SEQUENCE:  # type: ignore
             msg_id = codec.FMT_MESSAGE_ID.unpack_from(
                 data, start + codec.LEN_HEADER + codec.LEN_REQUEST_RESPONSE
@@ -654,7 +667,13 @@ class RabbitMQStreamsProtocol(asyncio.Protocol):
         publisher_id, left, right = codec.decode_publish_confirm(
             data, start + codec.LEN_HEADER
         )
-        pm = self._published_messages.pop(publisher_id)
+        pm = self._published_messages.pop(publisher_id, None)
+        if not pm:
+            logger.warning(
+                'published messages for a publisher not found, publisher id={}'
+                .format(publisher_id)
+            )
+            return
 
         if left == pm.left and right == pm.right:
             pm.task.set_result(0)
